/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package jogo;

import abstrata.Jogador;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Random;
import javax.swing.JPanel;
import javax.swing.Timer;
import objetos.CanhaoDePlasma;
import objetos.Duke;
import objetos.Inimigo;
import objetos.Puffy;
import objetos.Tux;
import objetos.Chefao;
import objetos.PotenciaCanhaoDePlasma;
import objetos.Velocidade;

public class Tela extends JPanel implements ActionListener {

    public Timer timer; // para ficar desenhando a Tela2 de tempos em tempos
    private Random r; // só para colocar os jogadores em posições aleatórias na tela
    public boolean jogoAtivo; // para saber se o jogo está rodando
    public boolean inicial; // para saber se é um novo jogo
    private Jogo pai;
    private Jogador personagemPrincipal;
    private boolean ativarChefao;   // para ativar/desativar o chefão
    public ArrayList<Inimigo> inimigos; // vetor de inimigos
    public ArrayList<Chefao> chefoes;   // vetor de chefões
    public CanhaoDePlasma canhaodeplasma;
    private int quantidadeInimigos; // quantidade de inimigos a ser criada
    private int velocidadeInimigo = 1;  // velocidade inicial do inimigo
    private int tempoDoJogo;    // tempo usado para o inimigo disparar canhão
    private int tempoDoChefao;  // tempo usado para o chefão disparar canhão
    private double duracaoEscudoForca;
    private int quantidadeDeTirosInimigo = 2; // quantidade de tiros inicial do inimigo
    private ArrayList<Velocidade> velocidade; // vetor de powerup's de velocidade
    private ArrayList<PotenciaCanhaoDePlasma> potenciacanhao; // vetor de powerup's de potência do canhão
    private boolean powerupVelocidade = false; // powerup de velocidade
    private boolean powerupPotenciaCanhao = false;  // powerup de pontência do canhão
    private int velocidadeChefao = 3; // velocidade inicial do chefão
    private int limiteDisparosChefao = 5; // quantidade de disparos inicial do chefão
    private int vidaChefao = 0; // vida inicial do chefão
    private int velocidadeVertical = 2; // velocidade vertical inicial do inimigo

    public Tela(Jogo p) {
        // Guardando um referencia ao JFrame PAI para pegar e modificar
        // alguns atributos que lá estão
        this.pai = p;
        // tempo usado para disparos do canhão inimigo
        tempoDoJogo = 0;
        // tempo usado para disparos do canhão do chefão
        tempoDoChefao = 0;
        this.r = new Random();
        // atributos que poderão ser modificados pela classe Principal
        this.jogoAtivo = true;
        this.inicial = false;
        // trata eventos das teclas pressionadas
        this.addKeyListener(new TratadorEventoTeclado());
        /* definindo algumas propriedades do JPanel */
        this.setFocusable(true); // assume foco
        this.setDoubleBuffered(true); // tornando o redesenho mais rapido
        /* Irá atualizar a tela a cada 10 milisegundos */
        this.timer = new Timer(10, this);
        // iniciando o timer
        this.timer.start();
        this.inimigos = new ArrayList<Inimigo>();
        this.chefoes = new ArrayList<Chefao>();
        this.velocidade = new ArrayList<Velocidade>();
        this.potenciacanhao = new ArrayList<PotenciaCanhaoDePlasma>();
        this.ativarChefao = false;
    }

    /**
     * É chamado sempre que começar uma nova rodada
     */
    public void novaRodada() {
        // coordenadas do jogador principal
        int px = 10;
        int py = this.getHeight() / 2;
        /* criando jogador principal e posicionando na tela */
        if (this.pai.personagemEscolhido.equals("tux")) {
            personagemPrincipal = new Tux("/imagens/tux.png", this, px, py, this.pai);
        } else if (this.pai.personagemEscolhido.equals("duke")) {
            personagemPrincipal = new Duke("/imagens/duke.png", this, px, py, this.pai);
        } else if (this.pai.personagemEscolhido.equals("puffy")) {
            personagemPrincipal = new Puffy("/imagens/puffy.png", this, px, py, this.pai);
        }
        // definindo quantidade de inimigos aleatoriamente, minimo 1 maximo 5
        quantidadeInimigos = r.nextInt(5) + 1;
        // criando inimigos
        // olhar metodo
        criarInimigos(quantidadeInimigos);
        // duracao inicial do escudo de forca (invencibilidade)
        duracaoEscudoForca = 0;
    }

    /**
     * Irá desenhar os objetos na tela
     * Será chamado a cada 10 milisegundos, ou seja, a tela está em constante
     * atualização
     * @param g
     */
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;

        if (jogoAtivo) { // o jogo está rodando!!!
            // Jogo acabou de ser carregado, mas ainda não foi iniciado um novo jogo
            // para criar os objetos somente 1 vez.
            if (inicial == true) {
                this.novaRodada(); //olhar dentro do metodo
                inicial = false;
            }
            // verifcando se o personagem principal não é nulo
            if (personagemPrincipal != null) {
                // incrementa o tempo, de 1 a 99
                tempoDoJogo = (tempoDoJogo + 1) % 100;
                // incrementa o tempo, de 1 a 99
                tempoDoChefao = (tempoDoChefao + 1) % 100;
                // desenhando o personagem principal
                g2d.drawImage(personagemPrincipal.getImagem(), personagemPrincipal.getX(), personagemPrincipal.getY(), this);
                // percorrendo o vetor de tiros horizontal e desenhando na tela
                ArrayList<CanhaoDePlasma> cp = personagemPrincipal.obterCanhaoHorizontal();
                // percorrendo o vetor de tiros diagonal superior e desenhando na tela
                ArrayList<CanhaoDePlasma> cp1 = personagemPrincipal.obterCanhaoDiagonalSuperior();
                // percorrendo o vetor de tiros diagonal inferior e desenhando na tela
                ArrayList<CanhaoDePlasma> cp2 = personagemPrincipal.obterCanhaoDiagonalInferior();
                // desenhando os tiros horizontal do personagem
                for (CanhaoDePlasma c : cp) {
                    g2d.drawImage(c.getImagem(), c.getX(), c.getY(), this);
                }
                // desenhando os tiros diagonal superior do personagem
                for (CanhaoDePlasma c1 : cp1) {
                    g2d.drawImage(c1.getImagem(), c1.getX(), c1.getY(), this);
                }
                // desenhando os tiros diagonal inferior do personagem
                for (CanhaoDePlasma c2 : cp2) {
                    g2d.drawImage(c2.getImagem(), c2.getX(), c2.getY(), this);
                }
                // desenhando o chefão se ele for ativado
                if (ativarChefao == true) {
                    // vetor para remover os tiros do chefão caso haja colisão ou saia dos limites da tela
                    ArrayList<CanhaoDePlasma> vetorRemoverTiroChefao = new ArrayList<CanhaoDePlasma>();
                    for (Chefao boss : chefoes) {
                        // deenhando o chefão
                        g2d.drawImage(boss.getImagem(), boss.getX(), boss.getY(), this);
                        /* Máximo 5 disparos (boca)
                         * se o tempoDoJogo for igual ao r.nextInt(100) adiciona um disparo no vetor
                         */
                        if ((boss.canhaoChefaoBoca.size() < limiteDisparosChefao) && (tempoDoJogo == r.nextInt(100))) {
                            boss.canhaoChefaoBoca.add(new CanhaoDePlasma("/imagens/plasmac.png", this, boss.getX() - 10, boss.getY() + 50, velocidadeChefao + 2));
                        }
                        /* Máximo 5 disparos (pata superior)
                         * se o tempoDoJogo for igual ao r.nextInt(100) adiciona um disparo no vetor
                         */
                        if ((boss.canhaoChefaoPataSuperior.size() < limiteDisparosChefao) && (tempoDoJogo == r.nextInt(100))) {
                            boss.canhaoChefaoPataSuperior.add(new CanhaoDePlasma("/imagens/plasmac.png", this, boss.getX(), boss.getY() + 100, velocidadeChefao + 2));
                        }
                        /* Máximo 5 disparos (pata inferior)
                         * se o tempoDoJogo for igual ao r.nextInt(100) adiciona um disparo no vetor
                         */
                        if ((boss.canhaoChefaoPataInferior.size() < limiteDisparosChefao) && (tempoDoJogo == r.nextInt(100))) {
                            boss.canhaoChefaoPataInferior.add(new CanhaoDePlasma("/imagens/plasmac.png", this, boss.getX(), boss.getY(), velocidadeChefao + 2));
                        }
                        // Percorrendo vetor de disparos (boca)
                        for (CanhaoDePlasma cC : boss.canhaoChefaoBoca) {
                            // desenhando disparos
                            g2d.drawImage(cC.getImagem(), cC.getX(), cC.getY(), this);
                            // movimenta o disparo se estiver dentro dos limites da tela
                            if (cC.visibilidadeTiro() == true) {
                                cC.dispararCanhaoDePlasma(false);
                            } else {
                                // descarta os tiros fora dos limites da tela
                                vetorRemoverTiroChefao.add(cC);
                            }
                        }
                        // Percorrendo vetor de disparos (pata superior)
                        for (CanhaoDePlasma cC : boss.canhaoChefaoPataSuperior) {
                            // desenhando disparos
                            g2d.drawImage(cC.getImagem(), cC.getX(), cC.getY(), this);
                            // movimenta o disparo se estiver dentro dos limites da tela
                            if (cC.visibilidadeTiro() == true) {
                                cC.dispararCanhaoDePlasma(false);
                            } else {
                                // descarta os tiros fora dos limites da tela
                                vetorRemoverTiroChefao.add(cC);
                            }
                        }
                        // Percorrendo vetor de disparos (pata inferior)
                        for (CanhaoDePlasma cC : boss.canhaoChefaoPataInferior) {
                            // desenhando disparos
                            g2d.drawImage(cC.getImagem(), cC.getX(), cC.getY(), this);
                            // movimenta o disparo se estiver dentro dos limites da tela
                            if (cC.visibilidadeTiro() == true) {
                                cC.dispararCanhaoDePlasma(false);
                            } else {
                                // descarta os tiros fora dos limites da tela
                                vetorRemoverTiroChefao.add(cC);
                            }
                        }
                        // movimenta o chefão
                        boss.movimentarChefao();
                        // removendo disparos descartados do chefão (boca)
                        for (CanhaoDePlasma cC : vetorRemoverTiroChefao) {
                            boss.canhaoChefaoBoca.remove(cC);
                        }
                        // removendo disparos descartados do chefão (pata superior)
                        for (CanhaoDePlasma cC : vetorRemoverTiroChefao) {
                            boss.canhaoChefaoPataSuperior.remove(cC);
                        }
                        // removendo disparos descartados do chefão (pata inferior)
                        for (CanhaoDePlasma cC : vetorRemoverTiroChefao) {
                            boss.canhaoChefaoPataInferior.remove(cC);
                        }
                    }
                }
                // percorrendo vetor de inimigos e desenhando na tela
                int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
                for (Inimigo ini : inimigos) {
                    ArrayList<CanhaoDePlasma> vetorRemoverTiroInimigo = new ArrayList<CanhaoDePlasma>();
                    // desenhando o inimigo
                    g2d.drawImage(ini.getImagem(), ini.getX(), ini.getY(), this);
                    Font small = new Font("Helvetica", Font.CENTER_BASELINE, 12);
                    g2d.setColor(Color.black);
                    g2d.setFont(small);
                    String msg = "" + ini.vidaInimigo;
                    // desenhando a vida do inimigo 
                    g2d.drawString(msg, ini.getX() + ini.getLargura(), ini.getY());
                    // cada inimigo poderá disparar duas vezes por tela ou antes de colisão com jogador
                    if ((ini.canhaoInimigo.size() < quantidadeDeTirosInimigo) && (tempoDoJogo == r.nextInt(100))) {
                        ini.canhaoInimigo.add(new CanhaoDePlasma("/imagens/laseri.png", this, ini.getX() - ini.getLargura(), ini.getY() + ini.getAltura() / 2 - 7, ini.velocidadeInimigo + 2));
                    }
                    // desenhando os disparos do inimigos e movimentando na tela
                    for (CanhaoDePlasma cI : ini.canhaoInimigo) {
                        g2d.drawImage(cI.getImagem(), cI.getX(), cI.getY(), this);
                        // dispara se está dentro dos limites da tela
                        if (cI.visibilidadeTiro() == true) {
                            cI.dispararCanhaoDePlasma(false);
                        } else {
                            // descatar se estiver fora dos limites da tela
                            vetorRemoverTiroInimigo.add(cI);
                        }
                    }
                    // movimentando inimigos
                    ini.movimentarInimigo();
                    // removendo disparos descartados dos inimigos
                    for (CanhaoDePlasma cI : vetorRemoverTiroInimigo) {
                        ini.canhaoInimigo.remove(cI);
                    }
                }
                // cria novos inimigos sempre que o vetor de inimigos for menor que 3
                if (inimigos.size() < 3) {
                    int totalInimigos = 5 - inimigos.size();
                    totalInimigos = r.nextInt(totalInimigos) + 1;
                    if (totalInimigos + inimigos.size() < 5) {
                        criarInimigos(totalInimigos);
                    }
                }
                // desenhando powerup de velocidade se estiver ativado
                if (powerupVelocidade == true) {
                    // percorrendo vetor de powerup de velocidade
                    for (Velocidade v : velocidade) {
                        // desenhando powerup de velocidade
                        g2d.drawImage(v.getImagem(), v.getX(), v.getY(), this);
                        // movimentando powerup de velocidade
                        v.moverVelocidade();
                    }
                }
                // desenhando powerup da potência do canhão se estiver ativado
                if (powerupPotenciaCanhao == true) {
                    // percorrendo vetor de powerup da potência do canhão
                    for (PotenciaCanhaoDePlasma p : potenciacanhao) {
                        // desenhando powerup da potência do canhão
                        g2d.drawImage(p.getImagem(), p.getX(), p.getY(), this);
                        // movimentando powerup da potência do canhão
                        p.moverPotenciaCanhaoDePlasma();
                    }
                }
                // verificando se o escudo de força está ativado
                if (personagemPrincipal.obterInvencibilidade() == true) {
                    // incrementa o tempo de duracao, de 0 a 99, atualizado
                    // a cada 10ms (repaint)
                    // tentativa de correção de bug
                    duracaoEscudoForca = (duracaoEscudoForca + 0.2) % 100;
                    pai.jProgressBarEscudoDeForça.setValue((int) (100 - duracaoEscudoForca));
                    // verificando se o tempo do escudo de força acabou
                    if (duracaoEscudoForca >= 99) {
                        // zerando o tempo do escudo de força
                        duracaoEscudoForca = 0;
                        pai.jProgressBarEscudoDeForça.setValue(0);
                        // desativando escudo de força
                        personagemPrincipal.setInvencibilidade(false);
                        // retorna à imagem original de acordo com o personagem
                        //  se for o Duke
                        if (personagemPrincipal instanceof Duke) {
                            personagemPrincipal.alterarImagem("/imagens/duke.png");
                        }
                        // se for o Puffy
                        if (personagemPrincipal instanceof Puffy) {
                            personagemPrincipal.alterarImagem("/imagens/puffy.png");
                        }
                    }
                }
                // verficando se a bomba aniquiladora foi ativada
                if (personagemPrincipal.obterAtivarBomba() == true) {
                    // vetor para remover os inimigos aniquilados
                    ArrayList<Inimigo> aniquilarInimigo = new ArrayList<Inimigo>();
                    // vetor para remover os disparos inimigos aniquilados
                    ArrayList<CanhaoDePlasma> aniquilarCanhaoInimigo = new ArrayList<CanhaoDePlasma>();
                    // percorrendo vetor de inimigos
                    for (Inimigo aI : inimigos) {
                        // adicionando inimigo no vetor para ser removido
                        aniquilarInimigo.add(aI);
                        // aumentando 1 ponto na pontuação para cada inimigo morto
                        pontos++;
                        // aumentando 5 pontos de poder para cada inimigo morto
                        pai.jProgressBarPoder.setValue(pai.jProgressBarPoder.getValue() + 5);
                        // atualizando pontuação
                        pai.jLabelPontos.setText("" + pontos);
                        // olhar método
                        aumentarAtributosInimigos();
                        // olhar método
                        criarChefão();
                        // se a pontuação for múltiplo de 6, um power up é criado
                        if (pontos % 6 == 0) {
                            /**
                             * Criação aleatória de powerup
                             * se for true, powerup de velocidade
                             * se for false, powerup de potência do canhão
                             */
                            boolean powerup = r.nextBoolean();
                            // powerup de velocidade
                            if (powerup == true) {
                                // cria powerup se a velocidade do personagem não estiver no máximo
                                if (pai.jProgressBarVelocidade.getValue() < 100) {
                                    // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                                    velocidade.add(new Velocidade("/imagens/velocidade.png", this, aI.getX(), aI.getY()));
                                    // ativando o powerup de velocidade
                                    powerupVelocidade = true;
                                    // se a velocidade estiver no máximo, tentará criar um powerup da potência do canhão
                                    // cria powerup de potência do canhão se não estiver no máximo
                                } else if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                                    // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                                    potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, aI.getX(), aI.getY()));
                                    // ativando o powerup da potência do canhão
                                    powerupPotenciaCanhao = true;
                                }
                            }
                            // powerup da potência do canhão
                            if (powerup == false) {
                                // cria powerup se a potência do canhão não estiver no máximo
                                if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                                    // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                                    potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, aI.getX(), aI.getY()));
                                    // ativando o powerup de potência do canhão
                                    powerupPotenciaCanhao = true;
                                    // se a potência do canhão estiver no máximo, tentará criar um powerup de velocidade
                                    // cria powerup de velocidade se não estiver no máximo
                                } else if (pai.jProgressBarVelocidade.getValue() < 100) {
                                    // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                                    velocidade.add(new Velocidade("/imagens/velocidade.png", this, aI.getX(), aI.getY()));
                                    // ativando o powerup de velocidade
                                    powerupVelocidade = true;
                                }
                            }
                            // descartando disparos inimigos
                            for (CanhaoDePlasma aCI : aI.canhaoInimigo) {
                                aniquilarCanhaoInimigo.add(aCI);
                            }
                        }
                    }
                    // removendo inimigos e disparos inimigos descartados
                    for (Inimigo aI2 : aniquilarInimigo) {
                        // removendo disparos inimigos
                        for (CanhaoDePlasma aCI2 : aniquilarCanhaoInimigo) {
                            aI2.canhaoInimigo.remove(aCI2);
                        }
                        // removendo inimigos
                        inimigos.remove(aI2);
                    }
                    // desativando a bomba aniquiladora
                    personagemPrincipal.ativarBombaAniquiladora(false);
                    // criando novos inimigos, mínimo 1, máximo 5
                    quantidadeInimigos = r.nextInt(5) + 1;
                    criarInimigos(quantidadeInimigos);
                }
                // para garantir que a atualização da tela seja síncrona
                Toolkit.getDefaultToolkit().sync();
                // liberando os recursos do sistema usados para gerar os gráficos acima
                g.dispose();
            }
        } else if (pai.jProgressBarVida.getValue() == 0) { // fim de jogo, imprima a mensagem
            // ativando o antialiasing
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            // modificando as características fonte
            Font small = new Font("Helvetica", Font.BOLD, 14);
            FontMetrics metr = this.getFontMetrics(small);
            g2d.setColor(Color.black);
            g2d.setFont(small);
            String msg = "Game Over";
            g2d.drawString(msg, (this.getWidth() - metr.stringWidth(msg)) / 2, this.getHeight() / 2);
            // olhar método
            prepararNovoJogo();
            // registrando recorde do jogador
            int recorde = new Integer(pai.jLabelRecordeMax.getText()).intValue();
            int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
            // atualiza recorde se pontuação for maior
            if (pontos > recorde) {
                pai.jLabelRecordeMax.setText("" + pontos);
            }
        }
    }

    /**
     * Se o jogo acabar ou for cancelado, reseta os valores atuais para iniciais
     */
    public void prepararNovoJogo() {
        // ativando item de menu
        pai.jMenuItemNovo.setEnabled(true);
        // desativando item de menu
        pai.jMenuItemCancelarJogo.setEnabled(false);
        // correção de bug
        pai.personagemEscolhido = "";
        personagemPrincipal.jogoAtivo = false;
        // valores iniciais
        velocidadeInimigo = 1;
        velocidadeVertical = 2;
        ativarChefao = false;
        quantidadeDeTirosInimigo = 2;
        personagemPrincipal.setPotenciaCanhao(0);
        pai.jProgressBarVelocidade.setValue(0);
        pai.jProgressBarPotenciaCanhao.setValue(0);
        pai.jProgressBarVidaChefao.setValue(0);
        pai.jProgressBarEscudoDeForça.setValue(0);
        vidaChefao = 0;
        velocidadeChefao = 3;
        limiteDisparosChefao = 5;
        // vetor para remover inimigos
        ArrayList<Inimigo> aniquilarInimigo = new ArrayList<Inimigo>();
        // vetor para remover disparos inimigos
        ArrayList<CanhaoDePlasma> aniquilarCanhaoInimigo = new ArrayList<CanhaoDePlasma>();
        for (Inimigo aI : inimigos) {
            // adicionando inimigos no vetor para ser removido
            aniquilarInimigo.add(aI);
            for (CanhaoDePlasma aCI : aI.canhaoInimigo) {
                // adicionando disparo inimigo no vetor para ser removido
                aniquilarCanhaoInimigo.add(aCI);
            }
        }
        // removendo inimigos e disparos inimigos
        for (Inimigo aI2 : aniquilarInimigo) {
            for (CanhaoDePlasma aCI2 : aniquilarCanhaoInimigo) {
                // removendo disparos inimigos
                aI2.canhaoInimigo.remove(aCI2);
            }
            // removendo inimigos
            inimigos.remove(aI2);
        }
        // vetor para remover chefão
        ArrayList<Chefao> aniquilarChefao = new ArrayList<Chefao>();
        // vetor para remover disparos do chefão (boca)
        ArrayList<CanhaoDePlasma> aniquilarCanhaoChefao1 = new ArrayList<CanhaoDePlasma>();
        // vetor para remover disparos do chefão (pata superior)
        ArrayList<CanhaoDePlasma> aniquilarCanhaoChefao2 = new ArrayList<CanhaoDePlasma>();
        // vetor para remover disparos do chefão (pata inferior)
        ArrayList<CanhaoDePlasma> aniquilarCanhaoChefao3 = new ArrayList<CanhaoDePlasma>();
        // adicionando disparos do chefão para remoção
        for (Chefao aC : chefoes) {
            // adicionando chefão para remoção
            aniquilarChefao.add(aC);
            // adicionando disparos (boca) para remoção
            for (CanhaoDePlasma aCC : aC.canhaoChefaoBoca) {
                aniquilarCanhaoChefao1.add(aCC);
            }
            // adicionando disparos (pata superior) para remoção
            for (CanhaoDePlasma aCC : aC.canhaoChefaoPataSuperior) {
                aniquilarCanhaoChefao2.add(aCC);
            }
            // adicionando disparos (pata inferior) para remoção
            for (CanhaoDePlasma aCC : aC.canhaoChefaoPataInferior) {
                aniquilarCanhaoChefao3.add(aCC);
            }
        }
        // removendo disparos do chefão (boca)
        for (Chefao aC2 : aniquilarChefao) {
            for (CanhaoDePlasma aCC2 : aniquilarCanhaoChefao1) {
                aC2.canhaoChefaoBoca.remove(aCC2);
            }
        }
        // removendo disparos do chefão (pata superior)
        for (Chefao aC3 : aniquilarChefao) {
            for (CanhaoDePlasma aCC3 : aniquilarCanhaoChefao2) {
                aC3.canhaoChefaoBoca.remove(aCC3);
            }
        }
        // removendo disparos do chefão (pata inferior)
        for (Chefao aC4 : aniquilarChefao) {
            for (CanhaoDePlasma aCC4 : aniquilarCanhaoChefao3) {
                aC4.canhaoChefaoBoca.remove(aCC4);
            }
        }
        // removendo chefão
        for (Chefao aC1 : aniquilarChefao) {
            chefoes.remove(aC1);
        }
        // vetor para remover powerup's de velocidade
        ArrayList<Velocidade> removerVelocidade = new ArrayList<Velocidade>();
        // vetor para remover powerup's de potência de canhão
        ArrayList<PotenciaCanhaoDePlasma> removerPotencia = new ArrayList<PotenciaCanhaoDePlasma>();
        // adicionando powerup de velocidade para remoção
        for (Velocidade v : velocidade) {
            removerVelocidade.add(v);
        }
        // adicionando powerup de potência do canhão para remoção
        for (PotenciaCanhaoDePlasma p : potenciacanhao) {
            removerPotencia.add(p);
        }
        // removendo powerup's de velocidade
        for (Velocidade v : removerVelocidade) {
            velocidade.remove(v);
        }
        // removendo powerup's de potência de canhão
        for (PotenciaCanhaoDePlasma p : removerPotencia) {
            potenciacanhao.remove(p);
        }
    }

    /**
     * Irá tratar todos os eventos que ocorrerem no JPanel.
     * O pressionar de uma tecla é um evento a ser tratado.
     *
     * @param e evento que ocorreu na Tela2
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        // para movimentar o jogador pela tela
        // somente depois que o objeto for criado, ou seja, depois de um novo jogo
        if (personagemPrincipal != null) {
            // movimentando jogador
            personagemPrincipal.mover();
            // vetor de disparos (horizontal) do personagem principal
            ArrayList<CanhaoDePlasma> cp = personagemPrincipal.obterCanhaoHorizontal();
            // vetor de disparos (diagonal superior) do personagem principal
            ArrayList<CanhaoDePlasma> cp1 = personagemPrincipal.obterCanhaoDiagonalSuperior();
            // vetor de disparos (diagonal inferior) do personagem principal
            ArrayList<CanhaoDePlasma> cp2 = personagemPrincipal.obterCanhaoDiagonalInferior();
            // vetor para remover disparos (horizontal) do personagem principal
            ArrayList<CanhaoDePlasma> tiroDescartado = new ArrayList<CanhaoDePlasma>();
            // vetor para remover disparos (diagonal superior) do personagem principal
            ArrayList<CanhaoDePlasma> tiroDescartado1 = new ArrayList<CanhaoDePlasma>();
            // vetor para remover disparos (diagonal inferior) do personagem principal
            ArrayList<CanhaoDePlasma> tiroDescartado2 = new ArrayList<CanhaoDePlasma>();
            // percorrendo vetor de disparos (horizontal) do personagem principal
            for (CanhaoDePlasma c : cp) {
                // dispara se estiver dentro dos limites da tela
                if (c.visibilidadeTiro() == true) {
                    c.dispararCanhaoDePlasma(true);
                } else {
                    tiroDescartado.add(c);
                }
            }
            // percorrendo vetor de disparos (diagonal superior) do personagem principal
            for (CanhaoDePlasma c1 : cp1) {
                // dispara se estiver dentro dos limites da tela
                if (c1.visibilidadeTiro() == true) {
                    c1.dispararCanhaoDePlasmaDiagonal(true);
                } else {
                    tiroDescartado1.add(c1);
                }
            }
            int indice2 = 0;
            // percorrendo vetor de disparos (diagonal inferior) do personagem principal
            for (CanhaoDePlasma c2 : cp2) {
                // dispara se estiver dentro dos limites da tela
                if (c2.visibilidadeTiro() == true) {
                    c2.dispararCanhaoDePlasmaDiagonal(false);
                    // descarta se estiver fora dos limites da tela
                } else {
                    tiroDescartado2.add(c2);
                }
                indice2++;
            }
            // removendo disparos descartados (horizontal)
            for (CanhaoDePlasma c : tiroDescartado) {
                cp.remove(c);
            }
            // removendo disparos descartados (diagonal superior)
            for (CanhaoDePlasma c1 : tiroDescartado1) {
                cp1.remove(c1);
            }
            // removendo disparos descartados (diagonal inferior)
            for (CanhaoDePlasma c2 : tiroDescartado2) {
                cp2.remove(c2);
            }
            // verficando colisões na tela
            this.verificaColisoes();
            // repintando a tela
            repaint();
        }
    }

    /**
     * Cria os inimigos iniciais e os sucessivos.
     * @param quantidade quantos inimigos serão criados após a morte de um
     */
    public void criarInimigos(int quantidade) {
        // cria novos inimigos
        for (int i = 0; i < quantidade; i++) {
            inimigos.add(new Inimigo("/imagens/joaninha.png", this, pai.jPanelTela.getWidth(), posicaoVertical(), velocidadeInimigo, velocidadeVertical));
        }
    }

    /**
     * Sempre que a pontuação for múltiplo de 6
     * a velocidade e quantidade de tiros dos inimigos aumentam
     */
    public void aumentarAtributosInimigos() {
        // pegando o valor atual da pontuação
        int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
        // aumentado atributos dos inimigos
        if (pontos % 6 == 0) {
            velocidadeInimigo++;
            quantidadeDeTirosInimigo++;
        }
    }

    /**
     * Cria o chefão sempre que a pontuação for múltiplo de 50
     */
    public void criarChefão() {
        int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
        // se a pontuação for múltiplo de 50, a velocidade, a vida e a quantidade
        // de disparos do chefão aumentam
        if ((pontos > 0) && (pontos % 50 == 0)) {
            velocidadeChefao += 2;
            limiteDisparosChefao += 5;
            vidaChefao += 1500;
        }
        // se a pontuação for múltiplo de 50, o chefão é criado se ele não estiver ativo
        if ((pontos > 0) && (pontos % 50 == 0) && (ativarChefao != true)) {
            ativarChefao = true;
            velocidadeChefao += 5;
            limiteDisparosChefao += 5;
            chefoes.add(new Chefao("/imagens/chefao.png", this, pai.jPanelTela.getWidth() - 195, pai.jPanelTela.getHeight() / 4, velocidadeChefao, vidaChefao));
            // aumentando o valor máximo da barra de progresso da vida do chefão
            pai.jProgressBarVidaChefao.setMaximum(vidaChefao);
            // carregando a vida do chefão
            pai.jProgressBarVidaChefao.setValue(chefoes.get(0).vidaChefao);
        }
    }

    /**
     * Verifica se houve colisoes entre os objetos na tela.
     */
    private void verificaColisoes() {
        // descobrindo as dimensões e posições dos jogadores
        Rectangle rJogadorPrincipal = personagemPrincipal.getLimites();
        // verificando colisão com powerup's de velocidade se estiver ativo
        if (powerupVelocidade == true) {
            // vetor para remover powerup's de velocidade
            ArrayList<Velocidade> removerVelocidade = new ArrayList<Velocidade>();
            // percorrendo vetor de powerup's de velocidade
            for (Velocidade v : velocidade) {
                // descobrindo dimensões do powerup
                Rectangle rVelocidade = v.getLimites();
                // caso haja colisão do personagem principal com o respectivo powerup
                if (rVelocidade.intersects(rJogadorPrincipal)) {
                    // aumenta a velocidade do personagem principal se não estiver no máximo
                    if (pai.jProgressBarVelocidade.getValue() < 100) {
                        // aumentando a velocidade do personagem principal
                        personagemPrincipal.incrementarVelocidadePersonagem(1);
                        // atualizando barra de progresso de velocidade
                        pai.jProgressBarVelocidade.setValue(pai.jProgressBarVelocidade.getValue() + 25);
                        // adicionando no vetor para remoção
                        removerVelocidade.add(v);
                    }
                    // descata powerup se a velocidade já estiver no máximo
                    removerVelocidade.add(v);
                }
            }
            // removendo powerup's de velocidade
            for (Velocidade v : removerVelocidade) {
                velocidade.remove(v);
                // desativa a animação de powerup's de velocidade se o tamanho do
                // respectivo vetor for zero
                if (velocidade.size() == 0) {
                    powerupVelocidade = false;
                }
            }
        }
        // verificando colisão powerup's de potência do canhão se estiver ativo
        if (powerupPotenciaCanhao == true) {
            // vetor para remover powerup's de potência de canhão
            ArrayList<PotenciaCanhaoDePlasma> removerPotencia = new ArrayList<PotenciaCanhaoDePlasma>();
            // percorrendo vetor de powerup's de potência do canhão
            for (PotenciaCanhaoDePlasma p : potenciacanhao) {
                // descobrindo dimensões do powerup
                Rectangle rPotencia = p.getLimites();
                // caso haja colisão do personagem principal com o respectivo powerup
                if (rPotencia.intersects(rJogadorPrincipal)) {
                    // aumenta velocidade e número de disparos (horizontal, diagonal superior e inferior)
                    // se não estiver no máximo
                    if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                        personagemPrincipal.incrementarVelocidadeTiro(1);
                        personagemPrincipal.incrementarNumeroDeDisparosHorizontal(1);
                        personagemPrincipal.incrementarNumeroDeDisparosDiagonalSuperior(1);
                        personagemPrincipal.incrementarNumeroDeDisparosDiagonalInferior(2);
                        personagemPrincipal.incrementarPotenciaCanhao(1);
                        // atualizando barra de progresso de potência do canhão
                        pai.jProgressBarPotenciaCanhao.setValue(pai.jProgressBarPotenciaCanhao.getValue() + 25);
                        // adicionando no vetor para remoção
                        removerPotencia.add(p);
                    }
                    // descarta powerup se a potência do canhão já estiver no máximo
                    removerPotencia.add(p);
                }
            }
            // removendo powerup's de potência do canhão
            for (PotenciaCanhaoDePlasma p : removerPotencia) {
                potenciacanhao.remove(p);
                // desativa animação de powerup's de potência do canhão se o tamanho do
                // respectivo vetor for zero
                if (potenciacanhao.size() == 0) {
                    powerupPotenciaCanhao = false;
                }
            }
        }
        // descobrindo as dimensões e posicões do chefao
        if (ativarChefao == true) {
            // vetor para remover disparos do chefão (boca)
            ArrayList<CanhaoDePlasma> tiroChefao1 = new ArrayList<CanhaoDePlasma>();
            // vetor para remover disparos do chefão (diagonal superior)
            ArrayList<CanhaoDePlasma> tiroChefao2 = new ArrayList<CanhaoDePlasma>();
            // vetor para remover disparos do chefão (diagonal inferior)
            ArrayList<CanhaoDePlasma> tiroChefao3 = new ArrayList<CanhaoDePlasma>();

            for (Chefao tirochefao : chefoes) {
                // percorrendo vetor de disparos do chefão (boca)
                for (CanhaoDePlasma cC : tirochefao.canhaoChefaoBoca) {
                    // descobrindo as dimensões do disparo do chefão
                    Rectangle rcC = new Rectangle(cC.getLimites());
                    // caso haja colisão com o personagem principal
                    if (rcC.intersects(rJogadorPrincipal)) {
                        // olhar método
                        colisaoCanhaoChefaoXJogador();
                        // adicionando no vetor para remoção
                        tiroChefao1.add(cC);
                    }
                }
                // removendo disparos (boca) atingidos
                for (CanhaoDePlasma ci : tiroChefao1) {
                    tirochefao.canhaoChefaoBoca.remove(ci);
                }
                // percorrendo vetor de disparos do chefão (pata superior)
                for (CanhaoDePlasma cC : tirochefao.canhaoChefaoPataSuperior) {
                    // descobrindo as dimensões do disparo do chefão
                    Rectangle rcC = new Rectangle(cC.getLimites());
                    // caso haja colisão com o personagem principal
                    if (rcC.intersects(rJogadorPrincipal)) {
                        // olhar método
                        colisaoCanhaoChefaoXJogador();
                        // adicionando no vetor para remoção
                        tiroChefao2.add(cC);
                    }
                }
                // removendo disparos (pata superior) atingidos
                for (CanhaoDePlasma ci : tiroChefao2) {
                    tirochefao.canhaoChefaoPataSuperior.remove(ci);
                }
                // percorrendo vetor de disparos do chefão (pata inferior)
                for (CanhaoDePlasma cC : tirochefao.canhaoChefaoPataInferior) {
                    // descobrindo as dimensões do disparo do chefão
                    Rectangle rcC = new Rectangle(cC.getLimites());
                    // caso haja colisão com o personagem principal
                    if (rcC.intersects(rJogadorPrincipal)) {
                        // olhar método
                        colisaoCanhaoChefaoXJogador();
                        // adicionando no vetor para remoção
                        tiroChefao3.add(cC);
                    }
                }
                // removendo disparos atingidos
                for (CanhaoDePlasma ci : tiroChefao3) {
                    tirochefao.canhaoChefaoPataInferior.remove(ci);
                }
            }
            // verficando a colisão do chefão com o personagem
            for (Chefao colisaoChefao : chefoes) {
                // descobrindo as dimensões do chefão
                Rectangle rChefao = new Rectangle(colisaoChefao.getLimites());
                // caso haja colisão com o personagem principal
                if (rChefao.intersects(rJogadorPrincipal)) {
                    // olhar método
                    colisaoJogadorXChefao();
                }
            }
        }
        // vetor para remover inimigos colidos com o personagem principal
        ArrayList<Inimigo> colidiram = new ArrayList<Inimigo>();
        // percorrendo vetor de inimigos
        for (Inimigo ini : inimigos) {
            // descobrindo dimensões dos inimigos
            Rectangle rInimigo = new Rectangle(ini.getLimites());
            // caso haja colisão com o personagem principal
            if (rJogadorPrincipal.intersects(rInimigo)) {
                // olhar método
                colisaoJogadorXInimigo();
                // verificando a pontuação atual
                int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
                // se a pontuação for múltiplo de 6, um powerup é criado
                if (pontos % 6 == 0) {
                    /**
                     * Criação aleatória de powerup
                     * se for true, powerup de velocidade
                     * se for false, powerup de potência do canhão
                     */
                    boolean powerup = r.nextBoolean();
                    // powerup de velocidade
                    if (powerup == true) {
                        // cria powerup se a velocidade do personagem não estiver no máximo
                        if (pai.jProgressBarVelocidade.getValue() < 100) {
                            // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                            velocidade.add(new Velocidade("/imagens/velocidade.png", this, ini.getX(), ini.getY()));
                            // ativando o powerup de velocidade
                            powerupVelocidade = true;
                            // se a velocidade estiver no máximo, tentará criar um powerup de potência do canhão
                            // cria powerup de potência do canhão se não estiver no máximo
                        } else if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                            // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                            potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, ini.getX(), ini.getY()));
                            // ativando o powerup de potência do canhão
                            powerupPotenciaCanhao = true;
                        }
                    }
                    // powerup de potência do canhão
                    if (powerup == false) {
                        // cria powerup de potência do canhão se não estiver no máximo
                        if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                            // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                            potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, ini.getX(), ini.getY()));
                            // ativando o powerup de potência do canhão
                            powerupPotenciaCanhao = true;
                            // se a potência do canhão estiver no máximo, tentará criar um powerup velocidade
                            // cria powerup de velocidade se não estiver no máximo
                        } else if (pai.jProgressBarVelocidade.getValue() < 100) {
                            // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                            velocidade.add(new Velocidade("/imagens/velocidade.png", this, ini.getX(), ini.getY()));
                            // ativando o powerup de velocidade
                            powerupVelocidade = true;
                        }
                    }
                }
                // adicionando inimigos colididos para remoção
                colidiram.add(ini);
            }
        }
        // removendo inimigos que sofreram colisão
        for (Inimigo i : colidiram) {
            inimigos.remove(i);
        }
        // vetor para remover disparos inimigos colididos com o personagem principal
        ArrayList<CanhaoDePlasma> tiroInimigo = new ArrayList<CanhaoDePlasma>();
        // percorrendo vetor de inimigos
        for (Inimigo ini : inimigos) {
            // percorrendo vetor de disparos de inimigos de cada inimigo
            for (CanhaoDePlasma cI : ini.canhaoInimigo) {
                // descobrindo as dimensões do disparo inimigo
                Rectangle rcI = new Rectangle(cI.getLimites());
                // caso haja colisão com o personagem principal
                if (rcI.intersects(rJogadorPrincipal)) {
                    // olhar método
                    colisaoCanhaoInimigoXJogador();
                    // adicionando disparo para ser removido
                    tiroInimigo.add(cI);
                }
            }
            // removendo disparos colididos
            for (CanhaoDePlasma ci : tiroInimigo) {
                ini.canhaoInimigo.remove(ci);
            }
        }
        // vetor para remover disparos (horizontal) do personagem principal
        ArrayList<CanhaoDePlasma> tiroDescartado = new ArrayList<CanhaoDePlasma>();
        // vetor para remover disparos (diagonal superior) do personagem principal
        ArrayList<CanhaoDePlasma> tiroDescartado1 = new ArrayList<CanhaoDePlasma>();
        // vetor para remover disparos (diagonal inferior) do personagem principal
        ArrayList<CanhaoDePlasma> tiroDescartado2 = new ArrayList<CanhaoDePlasma>();
        // vetor para remover inimigos
        ArrayList<Inimigo> inimigo = new ArrayList<Inimigo>();
        // vetor para remover chefão
        ArrayList<Chefao> chefao = new ArrayList<Chefao>();
        // percorrendo vetor de disparos (horizontal) do personagem principal
        for (CanhaoDePlasma cJ : personagemPrincipal.obterCanhaoHorizontal()) {
            // descobrindo dimensões do disparo do personagem principal
            Rectangle rCj = new Rectangle(cJ.getLimites());
            // percorrendo vetor de inimigos
            for (Inimigo ini : inimigos) {
                // descobrindo dimensões dos inimigos
                Rectangle rInimigo = new Rectangle(ini.getLimites());
                // caso haja colisão do disparo (horizontal) do personagem principal no inimigo
                if (rCj.intersects(rInimigo)) {
                    // adicionando disparo no vetor para remoção
                    tiroDescartado.add(cJ);
                    // tratando a colisão do disparo no inimigo
                    ini.vidaInimigo = ini.obterVidaInimigo() - cJ.obterDanoCanhaoDePlasma();
                    // adicionando inimigos no vetor para verificar a vida
                    inimigo.add(ini);
                }
            }
            // percorrendo vetor de chefão
            for (Chefao chef : chefoes) {
                // descobrindo dimensões do chefão
                Rectangle rChefao = new Rectangle(chef.getLimites());
                // caso haja colisão do disparo (horizontal) do personagem principal no chefão
                if (rChefao.intersects(rCj)) {
                    // adicionando disparo no vetor para remoção
                    tiroDescartado.add(cJ);
                    // tratando a colisão do disparo no chefão
                    chef.vidaChefao = chef.obterVidaChefao() - cJ.obterDanoCanhaoDePlasma();
                    // atualizando barra de progresso da vida do chefão
                    if (chef.obterVidaChefao() < 0) {
                        pai.jProgressBarVidaChefao.setValue((int) 0);
                        chef.vidaChefao = 0;
                    } else {
                        pai.jProgressBarVidaChefao.setValue(chef.obterVidaChefao());
                    }
                    // adicionando chefão no vetor para verificar a vida
                    chefao.add(chef);
                }
            }
        }
        // percorrendo vetor de disparos (diagonal superior) do personagem principal
        for (CanhaoDePlasma cJ : personagemPrincipal.obterCanhaoDiagonalSuperior()) {
            // descobrindo dimensões do disparo do personagem principal
            Rectangle rCj = new Rectangle(cJ.getLimites());
            // percorrendo vetor de inimigos
            for (Inimigo ini : inimigos) {
                // descobrindo dimensões dos inimigos
                Rectangle rInimigo = new Rectangle(ini.getLimites());
                // caso haja colisão do disparo (diagonal superior) do personagem principal no inimigo
                if (rCj.intersects(rInimigo)) {
                    // adicionando disparo no vetor para remoção
                    tiroDescartado1.add(cJ);
                    // tratando a colisão do disparo nos inimigos
                    ini.vidaInimigo = ini.obterVidaInimigo() - cJ.obterDanoCanhaoDePlasma();
                    // adicionando inimigos no vetor para verificar a vida
                    inimigo.add(ini);
                }
            }
            // percorrendo vetor de chefão
            for (Chefao chef : chefoes) {
                // descobrindo dimensões do chefão
                Rectangle rChefao = new Rectangle(chef.getLimites());
                // caso haja colisão do disparo (diagonal superior) do personagem principal no chefão
                if (rChefao.intersects(rCj)) {
                    // adicionando disparo no vetor para remoção
                    tiroDescartado1.add(cJ);
                    // tratando a colisão do disparo no chefão
                    chef.vidaChefao = chef.obterVidaChefao() - cJ.obterDanoCanhaoDePlasma();
                    // atualizando barra de progresso da vida do chefão
                    if (chef.obterVidaChefao() < 0) {
                        pai.jProgressBarVidaChefao.setValue((int) 0);
                        chef.vidaChefao = 0;
                    } else {
                        pai.jProgressBarVidaChefao.setValue(chef.obterVidaChefao());
                    }
                    // adicionando chefão no vetor para verificar a vida
                    chefao.add(chef);

                }
            }
        }
        // percorrendo vetor de disparos (diagonal inferior) do personagem principal
        for (CanhaoDePlasma cJ : personagemPrincipal.obterCanhaoDiagonalInferior()) {
            // descobrindo dimensões do disparo do personagem principal
            Rectangle rCj = new Rectangle(cJ.getLimites());
            // percorrendo vetor de inimigos
            for (Inimigo ini : inimigos) {
                // descobrindo dimensões dos inimigos
                Rectangle rInimigo = new Rectangle(ini.getLimites());
                // caso haja colisão do disparo (diagonal inferior) do personagem principal no inimigo
                if (rCj.intersects(rInimigo)) {
                    // adicionando disparo no vetor para remoção
                    tiroDescartado2.add(cJ);
                    // tratando a colisão do disparo nos inimigos
                    ini.vidaInimigo = ini.obterVidaInimigo() - cJ.obterDanoCanhaoDePlasma();
                    // adicionando inimigos no vetor para verificar a vida
                    inimigo.add(ini);
                }
            }
            // percorrendo vetor de chefão
            for (Chefao chef : chefoes) {
                // descobrindo dimensões do chefão
                Rectangle rChefao = new Rectangle(chef.getLimites());
                // caso haja colisão do disparo (diagonal inferior) do personagem principal no chefão
                if (rChefao.intersects(rCj)) {
                    // adicionando disparo no vetor para remoção
                    tiroDescartado2.add(cJ);
                    // tratando a colisão do disparo no chefão
                    chef.vidaChefao = chef.obterVidaChefao() - cJ.obterDanoCanhaoDePlasma();
                    // atualizando barra de progresso da vida do chefão
                    if (chef.obterVidaChefao() < 0) {
                        pai.jProgressBarVidaChefao.setValue((int) 0);
                        chef.vidaChefao = 0;
                    } else {
                        pai.jProgressBarVidaChefao.setValue(chef.obterVidaChefao());
                    }
                    // adicionando chefão no vetor para verificar a vida
                    chefao.add(chef);
                }
            }
        }
        // removendo disparos (horizontal) do personagem principal colididos com inimigos/chefão
        for (CanhaoDePlasma c : tiroDescartado) {
            personagemPrincipal.vetorCanhaoDePlasmaHorizontal.remove(c);
        }
        // removendo disparos (diagonal superior) do personagem principal colididos com inimigos/chefão
        for (CanhaoDePlasma c1 : tiroDescartado1) {
            personagemPrincipal.vetorCanhaoDePlasmaDiagonalSuperior.remove(c1);
        }
        // removendo disparos (diagonal inferior) do personagem principal colididos com inimigos/chefão
        for (CanhaoDePlasma c2 : tiroDescartado2) {
            personagemPrincipal.vetorCanhaoDePlasmaDiagonalInferior.remove(c2);
        }
        // percorrendo vetor de inimigos para verificação da vida
        for (Inimigo ini : inimigo) {
            // verificando se o inimigo morreu
            if (ini.vidaInimigo <= 0) {
                // aumentando 1 ponto na pontuação para cada inimigo morto
                int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
                pontos++;
                // atualizando a pontuação
                pai.jLabelPontos.setText("" + pontos);
                // aumentando 5 pontos de poder para cada inimigo morto
                pai.jProgressBarPoder.setValue(pai.jProgressBarPoder.getValue() + 5);
                // olhar método
                criarChefão();
                // se a pontuação for múltiplo de 6, um powerup é criado
                if (pontos % 6 == 0) {
                    // olhar método
                    aumentarAtributosInimigos();
                    /**
                     * Criação aleatória de powerup
                     * se for true, powerup de velocidade
                     * se for false, powerup de potência do canhão
                     */
                    boolean powerup = r.nextBoolean();
                    // powerup de velocidade
                    if (powerup == true) {
                        // cria powerup se a velocidade do personagem não estiver no máximo
                        if (pai.jProgressBarVelocidade.getValue() < 100) {
                            // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                            velocidade.add(new Velocidade("/imagens/velocidade.png", this, ini.getX(), ini.getY()));
                            // ativando o powerup de velocidade
                            powerupVelocidade = true;
                            // se a velocidade estiver no máximo, tentará criar um powerup de potência do canhão
                            // cria powerup de potência do canhão se não estiver no máximo
                        } else if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                            // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                            potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, ini.getX(), ini.getY()));
                            // ativando o powerup de potência do canhão
                            powerupPotenciaCanhao = true;
                        }
                    }
                    // powerup de potência do canhão
                    if (powerup == false) {
                        // cria powerup de potência do canhão se não estiver no máximo
                        if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                            // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                            potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, ini.getX(), ini.getY()));
                            // ativando o powerup de potência do canhão
                            powerupPotenciaCanhao = true;
                            // se a potência do canhão estiver no máximo, tentará criar um powerup velocidade
                            // cria powerup de velocidade se não estiver no máximo
                        } else if (pai.jProgressBarVelocidade.getValue() < 100) {
                            // cria o powerup na mesma coordenada x,y do inimigo que acabou de morrer
                            velocidade.add(new Velocidade("/imagens/velocidade.png", this, ini.getX(), ini.getY()));
                            // ativando o powerup de velocidade
                            powerupVelocidade = true;
                        }
                    }
                }
                // removendo inimigos mortos
                inimigos.remove(ini);
            }
        }
        // percorrendo vetor de chefão para verificação de vida
        for (Chefao chef : chefao) {
            // verificando se o chefão morreu
            if (chef.obterVidaChefao() <= 0) {
                // removendo chefão
                chefoes.remove(chef);
                pai.jProgressBarVidaChefao.setValue((int) 0);
                // aumentando 10 pontos na pontuação ao matar o chefão
                int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
                pontos += 10;
                // olhar método
                aumentarAtributosInimigos();
                // atualizando pontuação
                pai.jLabelPontos.setText("" + pontos);
                // aumentando 25 pontos de poder ao matar o chefão
                pai.jProgressBarPoder.setValue(pai.jProgressBarPoder.getValue() + 25);
                // desativando chefão
                ativarChefao = false;
            }
        }
    }

    /**
     * Trata a colisão do jogador principal com o inimigo
     */
    public void colisaoJogadorXInimigo() {
        int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
        // aumentando 1 ponto na pontuação para cada inimigo morto por colisão
        pontos++;
        // caso o escudo de força estiver desativado, o personagem principal sofrerá dano pela colisão
        if (personagemPrincipal.obterInvencibilidade() == false) {
            // ao ser colidido por um inimigo o personagem principal perde 25 pontos da vida
            pai.jProgressBarVida.setValue(pai.jProgressBarVida.getValue() - 25);
            // atualizando a pontuação
            pai.jLabelPontos.setText("" + pontos);
            // olhar método
            aumentarAtributosInimigos();
            // olhar método
            criarChefão();
            // se a vida do personagem principal zerar, o jogo acaba
            if (pai.jProgressBarVida.getValue() <= 0) {
                pai.jProgressBarVida.setValue(0);
                // desativando jogo
                jogoAtivo = false;
            }
            // caso o escudo de força estiver ativado
        } else {
            // atualizando a pontuação
            pai.jLabelPontos.setText("" + pontos);
            // olhar método
            aumentarAtributosInimigos();
            // olhar método
            criarChefão();
        }
    }

    /**
     * Trata a colisão do jogador principal com o chefão
     */
    public void colisaoJogadorXChefao() {
        // caso o escudo de força estiver desativado, o personagem principal sofrerá dano pela colisão
        if (personagemPrincipal.obterInvencibilidade() != true) {
            // ao ser colidido pelo chefão o personagem principal perde 50 pontos da vida
            int vida = pai.jProgressBarVida.getValue();
            vida -= 50;
            // se a vida do personagem principal zerar, o jogo acaba
            if (vida <= 0) {
                pai.jProgressBarVida.setValue(0);
                // desativando jogo
                jogoAtivo = false;
            } else {
                pai.jProgressBarVida.setValue(vida);
            }
        }
    }

    /**
     * Trata a colisão do tiro do chefão com o jogador
     */
    public void colisaoCanhaoChefaoXJogador() {
        // caso o escudo de força estiver desativado, o personagem principal sofrerá dano pela colisão
        if (personagemPrincipal.obterInvencibilidade() == false) {
            // cada disparo do chefão diminui 20 pontos da vida do jogador
            int vida = pai.jProgressBarVida.getValue();
            vida -= 20;
            // se a vida do personagem principal zerar, o jogo acaba
            if (vida <= 0) {
                pai.jProgressBarVida.setValue(0);
                // desativando jogo
                jogoAtivo = false;
            } else {
                pai.jProgressBarVida.setValue(vida);
            }
        }
    }

    /**
     *  Trata a colisão do tiro do inimigo com o jogador
     */
    public void colisaoCanhaoInimigoXJogador() {
        // caso o escudo de força estiver desativado, o personagem principal sofrerá dano pela colisão
        if (personagemPrincipal.obterInvencibilidade() == false) {
            // cada disparo dos inimigos diminui 10 pontos da vida do jogador
            int vida = pai.jProgressBarVida.getValue();
            vida -= 10;
            // se a vida do personagem principal zerar, o jogo acaba
            if (vida <= 0) {
                pai.jProgressBarVida.setValue(0);
                // desativando jogo
                jogoAtivo = false;
            } else {
                pai.jProgressBarVida.setValue(vida);
            }
        }
    }

    /**
     * Gera a posição vertical de cada inimigo aleatoriamente
     * dentro dos limites do eixo Y
     * @return vertical posição do eixo Y gerada aleatoriamente
     */
    public int posicaoVertical() {
        int vertical = r.nextInt(pai.jPanelTela.getHeight());
        // se estiver fora dos limites do eixo Y, assume nova posição
        if (vertical > 264) {
            vertical = 264;
        }
        return vertical;
    }

    /**
     * Classe interna e privada para tratar os eventos de tecla pressionada
     * e tecla solta.
     * Neste exemplo, sempre que uma tecla for pressionada, o j1 irá se mover
     *
     * Uma classe interna consegue acessar todos os atributos (mesmo os privados)
     * da classe mais externa que a contém.
     *
     */
    private class TratadorEventoTeclado extends KeyAdapter {

        /**
         * Quando ocorrer o evento de tecla solta, será invocado o método teclaSolta
         * do objeto personagem principal
         * @param e evento do teclado
         */
        @Override
        public void keyReleased(KeyEvent e) {
            if (personagemPrincipal != null) {
                personagemPrincipal.teclaSolta(e);
            }
        }

        /**
         * Quando ocorrer o evento de tecla pressionada, será invocado o
         * método teclaPressionada do objeto personagem principal
         * @param e evento do teclado
         */
        @Override
        public void keyPressed(KeyEvent e) {
            if (personagemPrincipal != null) {
                personagemPrincipal.teclaPressionada(e);
            }
        }
    }
}
