/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package objetos;

import abstrata.Personagem;
import java.util.Random;
import javax.swing.JPanel;

public class CanhaoDePlasma extends Personagem {

    private boolean visivel; // visibilidade do tiro
    private int velocidadeTiro = 2; // velocidade inicial do disparo
    private Random r; // para gerar o dano do disparo do personagem principal
    private int dano; // armazena o valor do dano

    public CanhaoDePlasma(String nomeImagem, JPanel jp, int px, int py, int velocidadeTiro) {
        super(nomeImagem, jp, px, py);
        // Cada disparo é criado com um dano mínimo de 1 e máximo 10
        r = new Random();
        this.dano = r.nextInt(10) + 1;
        visivel = true;
        this.velocidadeTiro = velocidadeTiro;
    }

    /**
     * Retorna a visibilidade do tiro
     * @return visivel
     */
    public boolean visibilidadeTiro() {
        return visivel;
    }

    /**
     * Retorna o dano do canhao
     * @return dano
     */
    public int obterDanoCanhaoDePlasma() {
        return dano;
    }

    /**
     * Movimenta o disparo na tela de acordo com o objeto - personagem e inimigo
     * @param direcao Se for true, desloca para direita, falso, para esquerda.
     */
    public void dispararCanhaoDePlasma(boolean direcao) {
        if (direcao == true) {
            x += velocidadeTiro;
            // se ultrapassar o limite do eixo X é descartado
            if (x > tela.getWidth()) {
                visivel = false;
            }
        } else {
            x -= velocidadeTiro;
            // se ultrapassar o limite do eixo X é descartado
            if (x < 0) {
                visivel = false;
            }
        }
    }

    /**
     * Movimenta o disparo na tela de acordo com com a direção
     * @param direcao Se for true, desloca diagonal superior, falso, diagonal inferior.
     */
    public void dispararCanhaoDePlasmaDiagonal(boolean direcao) {
        if (direcao == true) {
            x += velocidadeTiro;
            y -= velocidadeTiro / 3;
            // se ultrapassar o limite do eixo X ou Y é descartado
            if ((x > tela.getWidth()) || (y < 0)) {
                visivel = false;
            }
        } else {
            x += velocidadeTiro;
            y += velocidadeTiro / 3;
            // se ultrapassar o limite do eixo X ou Y é descartado
            if ((x > tela.getWidth()) || (y > tela.getHeight())) {
                visivel = false;
            }
        }
    }
}
