/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package objetos;

import abstrata.Personagem;
import java.util.ArrayList;
import java.util.Random;
import javax.swing.JPanel;

public class Chefao extends Personagem {

    private Random r; // para gerar movimentação aleatória do chefão
    public int vidaChefao; // vida do chefão
    private boolean visivel;
    public int velocidadeChefao; // velocidade de movimentação do chefão
    public ArrayList<CanhaoDePlasma> canhaoChefaoBoca; // vetor de disparos (boca)
    public ArrayList<CanhaoDePlasma> canhaoChefaoPataSuperior; // vetor de disparos (pata superior)
    public ArrayList<CanhaoDePlasma> canhaoChefaoPataInferior; // vetor de disparos (pata inferior)

    public Chefao(String nomeImagem, JPanel jp, int px, int py, int velocidade, int vida) {
        super(nomeImagem, jp, px, py);
        r = new Random();
        vidaChefao = vida;
        this.visivel = true;
        canhaoChefaoBoca = new ArrayList<CanhaoDePlasma>();
        canhaoChefaoPataSuperior = new ArrayList<CanhaoDePlasma>();
        canhaoChefaoPataInferior = new ArrayList<CanhaoDePlasma>();
        this.velocidadeChefao = velocidade;

    }

    /**
     * Retorna o valor da vida atual do chefão
     * @return vidaChefao
     */
    public int obterVidaChefao() {
        return vidaChefao;
    }

    /**
     * Algoritmo para movimentação horizontal e vertical do chefão
     */
    public void movimentarChefao() {
        // movimentaçao horizontal
        int posicaoX = r.nextInt(velocidadeChefao) + 1;
        boolean increX = r.nextBoolean();
        // se for true, incrementa a posição do eixo x
        if (increX == true) {
            if (getX() + getLargura() + posicaoX > tela.getWidth()) {
                setX(tela.getWidth() - getLargura());
            } else {
                incrementaX(posicaoX);
            }
        } // se for false, decrementa a posição do eixo x
        else {
            if (getX() - posicaoX > tela.getWidth() / 2) {
                incrementaX(posicaoX * -1);
            } else {
                setX(tela.getWidth() / 2);
            }
        }
        //movimentaçao vertical
        int posicaoY = r.nextInt(velocidadeChefao) + 1;
        boolean increY = r.nextBoolean();
        // se for true, incrementa a posição do eixo y
        if (increY == true) {
            if (getY() + getAltura() + posicaoY < tela.getHeight()) {
                incrementaY(posicaoY);
            } else {
                setY(tela.getHeight() - getAltura());
            }
        } // se for true, decrementa a posição do eixo y
        else {
            if (getY() - posicaoY > 0) {
                incrementaY(posicaoY * -1);
            } else {
                setY(0);
            }
        }
    }
}
